
#ifndef _FCITX5_GBIM_GBIM_H_
#define _FCITX5_GBIM_GBIM_H_

#include <iconv.h>

#include <fcitx-utils/inputbuffer.h>
#include <fcitx/addonfactory.h>
#include <fcitx/addonmanager.h>
#include <fcitx/inputcontext.h>
#include <fcitx/inputcontextproperty.h>
#include <fcitx/inputmethodengine.h>
#include <fcitx/instance.h>

class GBIMEngine;

class GBIMState : public fcitx::InputContextProperty {
public:
    GBIMState(GBIMEngine *engine, fcitx::InputContext *ic)
        : engine_(engine), ic_(ic) {}

    void keyEvent(fcitx::KeyEvent &keyEvent);
    void updateUI();
    void reset() {
        buffer_.clear();
        updateUI();
    }

private:
    static bool isHexLetterKey(const fcitx::Key &key);
    static bool shouldProcessCharKey(const fcitx::Key &key);

    size_t getMaxUserInputBufferSize() const;
    bool shouldUpdateCandidateWords() const;

private:
    GBIMEngine *engine_;
    fcitx::InputContext *ic_;
    fcitx::InputBuffer buffer_{{fcitx::InputBufferOption::AsciiOnly,
                                fcitx::InputBufferOption::FixedCursor}};
};

class GBIMEngine : public fcitx::InputMethodEngineV2 {
public:
    GBIMEngine(fcitx::Instance *instance);

    void activate(const fcitx::InputMethodEntry &entry,
                  fcitx::InputContextEvent &event) override;
    void keyEvent(const fcitx::InputMethodEntry &entry,
                  fcitx::KeyEvent &keyEvent) override;

    void reset(const fcitx::InputMethodEntry &,
               fcitx::InputContextEvent &event) override;

    auto factory() const { return &factory_; }
    auto conv() const { return conv_; }
    auto instance() const { return instance_; }

    FCITX_ADDON_DEPENDENCY_LOADER(quickphrase, instance_->addonManager());
    FCITX_ADDON_DEPENDENCY_LOADER(punctuation, instance_->addonManager());

private:
    FCITX_ADDON_DEPENDENCY_LOADER(chttrans, instance_->addonManager());
    FCITX_ADDON_DEPENDENCY_LOADER(fullwidth, instance_->addonManager());

    fcitx::Instance *instance_;
    fcitx::FactoryFor<GBIMState> factory_;
    iconv_t conv_;
};

class GBIMEngineFactory : public fcitx::AddonFactory {
    fcitx::AddonInstance *create(fcitx::AddonManager *manager) override {
        return new GBIMEngine(manager->instance());
    }
};

#endif // _FCITX5_GBIM_GBIM_H_
